#include <stdio.h>
#include "rdseed.h"

#define MAX_CASCADES 50				/* should be enough */

/* ----------------------------------------------------------------------- */

/* -------- prototypes ------------- */
float compute_gains();


/* ----------------------------------------------------------------------- */

void check_gains(t_50_ptr)
struct type50 *t_50_ptr;

{
	struct type52 *t_52_ptr;

	struct response *r;

	float gain_0 = 0;	/* overall gain */
	float gain_x[MAX_CASCADES];
	float computed_gains;
	int num_gains;	/* counter for gain not including the overall gain */
	BOOL found_gain_0 = 0;

	int i;

	while(t_50_ptr)
	{
		t_52_ptr = t_50_ptr->type52_head;

		while (t_52_ptr)
		{
			r = t_52_ptr->response_head;
		
			num_gains = 0;		/* new channel, reset some stuff */	
			gain_0 = 0;
			found_gain_0 = FALSE;
			memset((char *)gain_x, 0, sizeof(gain_x));

			while (r)
			{
				if (r->type == 'S')		/* type 58 */
				{
					if (r->ptr.type58->stage == 0)
					{
							gain_0 = r->ptr.type58->sensitivity;
							found_gain_0 = TRUE;
					}	
					else
						gain_x[num_gains++] = r->ptr.type58->sensitivity;

					if (num_gains >= MAX_CASCADES)
					{
						fprintf(stderr, "Unable to check gains, too many stages!\n");
				fprintf(stderr, "Unable to check gains, too many stages!  Network: %s; Station:%s; Channel: %s\n", 
						t_50_ptr->network_code,t_50_ptr->station, t_52_ptr->channel);
						r = r->next;
						continue;
					}

				}

				r = r->next;
			}			/* while responses */

			if (!found_gain_0 && r != t_52_ptr->response_head)
			{
				fprintf(stderr, "Unable to check gains, no gain for stage zero!  Network:%s; Station:%s; Channel: %s\n", 
					t_50_ptr->network_code,t_50_ptr->station, t_52_ptr->channel);
				t_52_ptr = t_52_ptr->next;
				continue;
			}
	
			if (num_gains > 0)
			{	
				computed_gains = compute_gains(gain_0, gain_x, num_gains);

				/* round off at .005 */
				round_off(&computed_gains);

				if (computed_gains > .005)
					fprintf(stderr, "Warning! (Stage 0 Gain  - (Gain_1 * Gain_2 * ...Gain_X)) / Stage 0 Gain is > than .005!\nNetwork:%s; Station:%s; Channel: %s; Computed value: %f\n\n", 
					t_50_ptr->network_code,t_50_ptr->station, t_52_ptr->channel, computed_gains);                   

			}

			t_52_ptr = t_52_ptr->next;

		}			/* while type 52s */

		t_50_ptr = t_50_ptr->next;
	
	}			/* while type 50s */


}

/* ------------------------------------------------------------------------ */
float compute_gains(g_0, g_x, n_g_x)
float g_0;
float g_x[];
int n_g_x;

{
	float g_x_product;

	int i;

	g_x_product = g_x[0];

	for (i = 1; i < n_g_x; i++)
		g_x_product *= g_x[i];	

	g_x_product = fabs(g_0 - g_x_product);

	return g_x_product / g_0;

}

/* ------------------------------------------------------------------------ */
void cmp_sample_rate();
/* ------------------------------------------------------------------------ */

check_decimations(t_50_ptr)
struct type50 *t_50_ptr;

{
    	struct type52 *t_52_ptr;
    	struct response *r;	

	float dec_sample_rate;

        while(t_50_ptr)
        {
 
    		t_52_ptr = t_50_ptr->type52_head;
 
    		while (t_52_ptr)
    		{
        		r = t_52_ptr->response_head;

			dec_sample_rate = 0;

			while (r)
			{
				if (r->type == 'D')		/* block 57 */
				{
					if (dec_sample_rate != 0)
					{

						cmp_sample_rate(dec_sample_rate, 
					    		r->ptr.type57->input_sample_rate,
					    		t_50_ptr,
					    		t_52_ptr);
					}

					dec_sample_rate = 
				   		r->ptr.type57->input_sample_rate/r->ptr.type57->decimation_factor;
				}

				if (r->type == 'R')		/* block 60 */
					/*decimate_47s(r->ptr.type60);*/
					decimate_47s(*r->ptr.type60);

				r = r->next;
	
			}

			t_52_ptr = t_52_ptr->next;

		}		/* while t_52_ptr */

		t_50_ptr = t_50_ptr->next;

	}		/* while t_50_ptr */
	
}
/* ------------------------------------------------------------------------ */
int decimate_47s(t_60)
struct type60 t_60;

{
	int i;

	float dec_sample_rate = 0;

	struct type47 *get_type47(), *t_47;

	for (i = 0; i < t_60.number_stages; i++)
	{
		t_47 = get_type47(&t_60.stage[i]);

		/* Not all type 60 references are to a blockette 47 */
		if (t_47 == NULL) 
			continue;	

                if (dec_sample_rate != 0)
                {
                        if ((int)dec_sample_rate != 
				(int)t_47->input_sample_rate)
                        {
                                fprintf(stderr, "Warning! check_decimations: detected bad sample rate decimation! Blockette 47! ");
                                fprintf(stderr, "for Network %s, Station %s, channel %s\n",
current_station->network_code, current_station->station, current_channel->channel);
                        }
                }
 
                dec_sample_rate =
                  t_47->input_sample_rate/t_47->decimation_factor;
		
	}


}

/* ------------------------------------------------------------------------ */
struct type47 *get_type47(stage)
struct type60sub1 *stage;


{

	struct type47 *t_47 = NULL, *get_47();

	int i;

	for (i = 0; i < stage->number_responses; i++)
    	{
		t_47 = get_47(stage->response[i].reference, type47_head);

		if (t_47 != NULL)
			return t_47;

	}
 
	return 0;
}


/* ------------------------------------------------------------------------ */
struct type47 *get_47(code, t_47)
int code;
struct type47 *t_47;
 
{
    while (t_47)
    {
        if (t_47->response_code == code)
            break;
 
        t_47 = t_47->next;
    }
 
    return t_47;
 
}
 
/* ------------------------------------------------------------------------ */
char *get_units_abbrev();
void cmp_units();
/* ------------------------------------------------------------------------ */

char *get_units_abbrev();

/* ------------------------------------------------------------------------ */
check_dimensions(t_50_ptr) 
struct type50 *t_50_ptr;
 
{

	struct type52 *t_52_ptr;
	struct response *r;

	char *prev_units, *input_units;


	while (t_50_ptr)
	{
		t_52_ptr = t_50_ptr->type52_head;
 
   		while (t_52_ptr)
		{
			r = t_52_ptr->response_head;
			prev_units = NULL;

			while (r)
        		{
        			switch (r->type)
				{
					case 'P' :	/* blockette 53 */
					case 'C' : 	/* blockette 54 */
						input_units = get_units_abbrev(r->ptr.type53->input_units_code);
				
						if (input_units == NULL)
						{
	fprintf(stderr, "ERROR - unable to locate dictionary reference for input units! Network %s; Station %s; channel %s; blockette 54\n", 
			t_50_ptr->network_code,t_50_ptr->station, t_52_ptr->channel);

							break;

						}
	
						if (prev_units != NULL)
							cmp_units(input_units, 
								  prev_units, 
								  t_50_ptr, 
								  t_52_ptr);

						prev_units = get_units_abbrev(r->ptr.type53->output_units_code);
					
						break;

					case 'L' : 	/* blockette 55 */
					case 'G' : 	/* clockette 56 */

				/* blockette 55&56 have the interesting data in the 
			 	 * same place, so can can get away with just referencing
			 	 * type55, the union math works out 
			 	 */
						input_units = 
							get_units_abbrev(r->ptr.type55->input_units_code);                        

						if (input_units == NULL)
                    				{
fprintf(stderr, "ERROR - unable to locate dictionary reference for input units! Network %s; Station %s; channel %s; blockette 55/56\n",
                            t_50_ptr->network_code,t_50_ptr->station, t_52_ptr->channel);
							break;
 
                    				}
	
                    				if (prev_units != NULL)     
                        				cmp_units(input_units, 
					  				prev_units, 
					  				t_50_ptr, t_52_ptr);     

                    				prev_units = 
							get_units_abbrev(r->ptr.type55->output_units_code);
 
						break;
		
          			}		/* switch */

				r = r->next;

			}		/* while response */	
		
			t_52_ptr = t_52_ptr->next;

		}		/* while t_52_ptr */

		t_50_ptr = t_50_ptr->next;

	}  		/* while t_50_ptr */

}
 
/* ------------------------------------------------------------------------ */
void cmp_units(in_units, prev_units, s_ptr, c_ptr)
char *in_units, *prev_units;
struct type50 *s_ptr;
struct type52 *c_ptr;

{
	if (strcmp(in_units, prev_units) != 0)
	{
		fprintf(stderr, "Warning! check_dimensions: detected a bad units cascade! Network %s; Station %s; Channel %s\n", 
			s_ptr->network_code, s_ptr->station, c_ptr->channel);

	}

	return;
}
 
/* ------------------------------------------------------------------------ */
int round_off(n)
float *n;

{
	*n *= 1000;	/* brings up to decimal pt */

	*n += .5;

	*n = (int)*n;

	*n = *n / 1000;

}

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
void cmp_sample_rate(r1, r2, s_ptr, c_ptr)
float r1, r2;
struct type50 *s_ptr;
struct type52 *c_ptr;

{

    if ((int)r1 != (int)r2)
    {
        fprintf(stderr, "Warning! check_decimations: detected bad sample rate decimation! Network %s; Station %s; Channel %s\n", 
s_ptr->network_code,s_ptr->station, c_ptr->channel);

    }

    return;
}

