/*===========================================================================*/
/* SEED reader     |             print_type47              |  station header */
/*===========================================================================*/
/*
	Name:		print_type47
	Purpose:	print the channel generic response table to standard output
	Usage:		void print_type47 ();
				print_type47 ();
	Input:		none - uses information in globally-available structure
	Output:		none
	Externals:	type47_head - 1st entry in table; defined in structures.h, 
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	print_response
	Calls to:	none
	Algorithm:	print the contents of the structure to the standard output.
	Notes:		The structure is a linked list; type47_head is the first member
				of the list
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill
				05/09/90  Sue Schoch      Initial release 2.0
*/

#include "rdseed.h"

int find_type47(fp, code)
FILE *fp;
int code;
{
	struct type47 *type47;							/* looping vbl */
	int i;										/* counter */

	for (type47 = type47_head; type47 != NULL; type47 = type47->next)
	{
		if (type47->response_code == code)
		{
			if (fp == NULL) break;

			fprintf (fp,"B047\n");
			fprintf (fp,"Response Lookup Code:                  %d\n",
				type47->response_code);
			fprintf (fp,"Response name:                         %s\n",
				type47->name);
			fprintf (fp,"Response input sample rate:            %E\n",
				type47->input_sample_rate );
			fprintf (fp,"Response decimation factor:            %d\n",
				type47->decimation_factor );
			fprintf (fp,"Response decimation offset:            %d\n",
				type47->decimation_offset );
			fprintf (fp,"Response delay:                        %E\n",
				type47->delay );
			fprintf (fp,"Response correction:                   %E\n",
				type47->correction );
			fprintf (fp,"\n");
			break;
		}
	}
	if (type47 == NULL) return(0); else return(1);
}
