/*===========================================================================*/
/* SEED reader     |             look_response             |  station header */
/*===========================================================================*/
/*
	Name:		look_response
	Purpose:	look through cirrent channel for matching response type
	Usage:		look_response ();
				char type;
				flag = look_response (type);
	Input:		type = a code letter indicating the type of response to recover
	Output:		Boolean true or false
	Externals:	current_channel = the current channel description being built
	Warnings:	none
	Errors:		none
	Called by:	parse_type53-59
	Calls to:	none
	Algorithm:	look through cirrent channel for matching response type
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Allen Nance
	Revisions:	7/30/92		Allen Nance
			23/3/99		Stephane zuzlewski
*/

#include "rdseed.h"

look_response (type, stage)
char type;
int stage;
{
	struct response *response;

	response = current_channel->response_head;
	while (response != NULL)
	{
		if (response->type == type)
		{
			switch (response->type)
			{
				case 'P':
					if (response->ptr.type53->stage == stage) return (TRUE);
					if (stage == -1) return(TRUE);
					break;
				case 'C':
					/* two consequetive 54's are OK */
					if (response->ptr.type54->stage == stage)
						if (response->next != NULL) return (TRUE);
					break;
				case 'L':
					if (response->ptr.type55->stage == stage) return (TRUE);
					break;
				case 'G':
					if (response->ptr.type56->stage == stage) return (TRUE);
					break;
				case 'D':
					if (response->ptr.type57->stage == stage) return (TRUE);
					break;
				case 'S':
					if (response->ptr.type58->stage == stage) return (TRUE);
					break;
				case 'F':
					if (response->ptr.type61->stage == stage) return (TRUE);
					break;
				case 'O':
					if (response->ptr.type62->stage == stage) return (TRUE);
					break;
			}
		}
		response = response->next;
	}

	return (FALSE);
}
