/*===========================================================================*/
/* SEED reader     |            parse_type100              |     data block  */
/*===========================================================================*/
/*
	Name:		parse_type100
	Purpose:	parse a data record header for blockette 100s.
	Usage:		void parse_type100 ();
				char *input_data_ptr;
				float *sample_rate;
				parse_type100 (input_data_ptr, &sample_rate);
	Input:		pointer to beginning of data record header
	Output:		none
	Externals:	none
	Warnings:	none
	Errors:		none
	Called by:	process_data
	Calls to:	none
	Algorithm:	search through data record blockettes for blockette 100 sample rate
	Notes:		none
	Problems:	none known
	References:	none
	Language:	C, hopefully ANSI standard
	Author:		Allen Nance
	Revisions:	09/18/92  Allen Nance  Initial version
*/

#include "rdseed.h"


void parse_type100 (input_data_ptr, sample_rate)
char *input_data_ptr;							/* ptr to start of data record */
float *sample_rate;							/* ptr to returned sample rate */
{
	struct input_data_hdr *input_data_hdr;				/* fixed data header */
	struct data_blk_100 *blk_100;					/* blockette 100 pointer */
	int i;								/*counter */

	/* point to beginning data header structure */
	input_data_hdr = (struct input_data_hdr *) input_data_ptr;

	blk_100 = (struct data_blk_100 *) &input_data_hdr->bod;
/*	printf("data blk next %d\n", blk_100->hdr.next_blk_byte); */

	while (blk_100->hdr.next_blk_byte != 0)
	{
		if ( ((blk_100->hdr.next_blk_byte%4)!=0) ||
		     (blk_100->hdr.next_blk_byte < (sizeof(struct input_data_hdr)+8)) )
		{
			fprintf (stderr, "WARNING (parse_type100):  ");
			fprintf (stderr, "invalid byte pointer = %d\n", blk_100->hdr.next_blk_byte);
			fprintf (stderr, "\tData blockette parsing aborted.\n");
			break;
		}
		blk_100 = (struct data_blk_100 *) (input_data_ptr+(blk_100->hdr.next_blk_byte-8));
		if (blk_100->hdr.type == 100)
		{
			*sample_rate = blk_100->sample_rate;

			if (*sample_rate == 0)
			{
				char this_station[10], this_channel[5], 
						this_network[3], this_location[3];
	
				int i;

				/* recover station name */
        			for (i = 0; i < 5 && (input_data_hdr->station[i] != ' '); i++)
                			this_station[i] = input_data_hdr->station[i];

        			this_station[i] = '\0';

				/* recover channel name */
				for (i = 0; i < 3 && (input_data_hdr->channel[i] != ' '); i++)
					this_channel[i] = input_data_hdr->channel[i];

				this_channel[i] = '\0';

				/* recover network name */
                                for (i = 0; i < 2 && (input_data_hdr->network[i] != ' '); i++)
                                        this_network[i] = input_data_hdr->network[i];
 
                                this_network[i] = '\0';
 
				/* location code - May have spaces */
				for (i = 0; i < 2; i++)
					this_location[i] = input_data_hdr->location[i];

				this_location[2] = '\0';

				for (i = 1; this_location[i] == ' '; i--)
				{
					this_location[i] = '\0';
					if (i == 0)
						break;

				}

				log_tspan_error(this_station,
                                		this_channel,
                                		this_network,
                                		this_location,
                                		&input_data_hdr->time,
                                		"ERROR: Block 100 has sample rate of 100");

				if (sniff_flag > 0)
                		{
                       
					fprintf(stderr, "ERROR - found Block 100 with sample rate of Zero\n"); 
					fprintf(stderr, "\tfor data record:stn/chn/net/loc: %s, %s, %s, %s\n",
                                        			this_station,
                                        			this_channel,
                                        			this_network,
                                        			this_location);

                        		fprintf(stderr, "\t\tat: %d, %d, %d:%d:%d.%d\n",
                                        		input_data_hdr->time.year,
                                        		input_data_hdr->time.day,
                                        		input_data_hdr->time.hour,
                                        		input_data_hdr->time.minute,
                                        		input_data_hdr->time.second,
                                        		input_data_hdr->time.fracsec);
 
                		}       /* if sniff_flag > 0 */
			}

		}
	}

}
