#include "sanity.h"
#include <stdio.h>
#include <stdlib.h>


extern double version;
/*double version = (double)VERSION;*/


int check_blockette_30(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;

    int i;
    int number_of_keys;

    char error_string[256];

    struct result_struct result;

    result.blockette_number = 30;
    strncpy(result.blockette_name, "Data Format Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 030", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 18, -1, &result);
    if (blockette_length < 18) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_string_mask(&blockette_pointer[7], 3, "Short descriptive name", 50, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 4, "Data format identifier code", 4, 0, 0, 0, &result);
    byte_number += 4;

    if (check_number_mask(&blockette_pointer[byte_number], 5, "Data family type", 3, 0, 0, 0, &result)) {
        if (!check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)0, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)90, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)91, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)1, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)50, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)80, 0, &result)
	    && !check_number_equal_to_value(&blockette_pointer[byte_number], 5, "Data family type", 3, (double)81, 0, &result)) {
	    
    	    sprintf(error_string, "The value %i is not equal to one of the 7 values that \n\t\tare accepted for this field (0, 1, 50, 80, 81, 90, 91)", return_number(&blockette_pointer[byte_number], 3));
	    add_error_to_result(&blockette_pointer[byte_number], 5, "Data family type", 3, 1, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Data family type", 3, 0, error_string, &result);
    }
    byte_number += 3;

    if (check_number_mask(&blockette_pointer[byte_number], 6, "Number of decoder keys", 2, 0, 0, 0, &result)) {
        number_of_keys = return_number(&blockette_pointer[byte_number], 2);
	byte_number += 2;

	for(i=0; i < number_of_keys; ++i) {
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 7, "Decoder keys", blockette_length - byte_number, 1, "UNLPS", &result);
	    byte_number += bytes_function_read;
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field and the Decoder keys were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Number of decoder keys", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_31(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;
    char error_string[256];


    struct result_struct result;

    result.blockette_number = 31;
    strncpy(result.blockette_name, "Comment Description Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 031", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 17, 86, &result);
    if (blockette_length < 17) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Comment code key", 4, 0, 0, 0, &result);
    byte_number += 4;
    
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Comment class code", 1, 0, "U", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Description of comment", 70, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 6, "Units of comment level", 3, 0, 0, 0, &result);
    byte_number += 3;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_32(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];


    struct result_struct result;

    result.blockette_number = 32;
    strncpy(result.blockette_name, "Cited Source Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;

    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 032", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 15, 202, &result);
    if (blockette_length < 15) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    check_number_mask(&blockette_pointer[byte_number], 3, "Source lookup code", 2, 0, 0, 0, &result);
    byte_number += 2;
    
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Name of publication/author", 70, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Date published/catalog", 70, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 6, "Publisher name", 50, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_33(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];


    struct result_struct result;

    result.blockette_number = 33;
    strncpy(result.blockette_name, "Generic Abbreviation Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 033", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 12, 61, &result);
    if (blockette_length < 12) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    check_number_mask(&blockette_pointer[byte_number], 3, "Abbreviation lookup code", 3, 0, 0, 0, &result);
    byte_number += 3;
    
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Abbreviation description", 50, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_34(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];


    struct result_struct result;

    result.blockette_number = 34;
    strncpy(result.blockette_name, "Units Abbreviations Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 034", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 13, 82, &result);
    if (blockette_length < 13) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    check_number_mask(&blockette_pointer[byte_number], 3, "Unit lookup code", 3, 0, 0, 0, &result);
    byte_number += 3;
    
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Unit name", 20, 1, "UNP", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Unit description", 50, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_35(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int number_of_components;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 35;
    strncpy(result.blockette_name, "Beam Configuration Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 035", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 14, -1, &result);
    if (blockette_length < 14) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Beam lookup code", 3, 0, 0, 0, &result);
    byte_number += 3;

    if(check_number_mask(&blockette_pointer[byte_number], 4, "Number of components", 4, 0, 0, 0, &result)) {
        number_of_components = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - 14) % 19 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 5 - 9 repeat %i times, while they should repeat %i times", number_of_components, (blockette_length - 14) / 19, number_of_components);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 4, "Number of components", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 5 - 9 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 4, "Number of components", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < number_of_components; ++i) {
	        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Station identifier", 5, 0, "UN", &result);
		byte_number += bytes_function_read;

	        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 6, "Location identifier", 2, 0, "UN", &result);
		byte_number += bytes_function_read;

	        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 7, "Channel identifier", 3, 0, "UN", &result);
		byte_number += bytes_function_read;

		check_number_mask(&blockette_pointer[byte_number], 8, "Sub-channel identifier", 4, 0, 0, 0, &result);
		byte_number += 4;

		check_number_mask(&blockette_pointer[byte_number], 9, "Component weight", 5, 2, 0, 0, &result);
		byte_number += 5;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field, and fields 5 - 9 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Number of decoder keys", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_41(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int lookup_key;
    int number_of_factors;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 41;
    strncpy(result.blockette_name, "FIR Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 041", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;
    
    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 24, -1, &result);
    if (blockette_length < 24) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result)) {
        lookup_key = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;
	
	if (lookup_key == 0) {
	    sprintf(error_string, "The Response Lookup Key cannot equal 0");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 3, "Response Lookup Key", 4, 1, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because of the above error, field 3 could not be checked");
	add_error_to_result(&blockette_pointer[byte_number - 4], 3, "Response Lookup Key", 4, 0, error_string, &result);
    }


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response Name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'C', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, or C)");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 6, "Signal In Units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 7, "Signal Out Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    if(check_number_mask(&blockette_pointer[byte_number], 8, "Number of Factors", 4, 0, 0, 0, &result)) {
        number_of_factors = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 14 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tField 9 repeats %i times, while it should repeat %i times", number_of_factors, (blockette_length - byte_number) / 14, number_of_factors);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 8, "Number of Factors", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, field 9 was not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 8, "Number of Factors", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < number_of_factors; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 9, "FIR Coefficient", 14, 3, 11, 1, &result);
		byte_number += 14;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of field 9 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Number of Factors", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_42(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int lookup_key;
    int number_of_coefficients;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 42;
    strncpy(result.blockette_name, "Polynomial Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 042", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;
    
    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 24, -1, &result);
    if (blockette_length < 24) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result)) {
        lookup_key = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;
	
	if (lookup_key == 0) {
	    sprintf(error_string, "The Response Lookup Key cannot equal 0");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 3, "Response Lookup Key", 4, 1, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because of the above error, field 3 could not be checked");
	add_error_to_result(&blockette_pointer[byte_number - 4], 3, "Response Lookup Key", 4, 0, error_string, &result);
    }


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response Name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Transfer Function Type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 5, "Transfer Function Type", 1, 'P', 0, &result))    {

        sprintf(error_string, "The character in the field is not one of the accepted value (P)");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Transfer Function Type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 6, "Signal In Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    check_number_mask(&blockette_pointer[byte_number], 7, "Signal Out Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 8, "Polynomial Approximation Type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 8, "Polynomial Approximation Type", 1, 'M', 0, &result))    {

        sprintf(error_string, "The character in the field is not one of the accepted value (M)");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Polynomial Approximation Type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 9, "Valid Frequency Units", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 9, "Valid Frequency Units", 1, 'A', 0, &result)
	&& (!check_char_value(&blockette_pointer[byte_number], 9, "Valid Frequency Units", 1, 'B', 0, &result))) {
        sprintf(error_string, "The character in the field is not one of the accepted value (A or B)");
	add_error_to_result(&blockette_pointer[byte_number], 9, "Valid Frequency Units", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 10, "Lower Valid Frequency Bound", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 11, "Upper Valid Frequency Bound", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 12, "Lower bound of approximation", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 13, "Upper bound of approximation", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 14, "Maximum Absolute Error", 12, 3, 9, 1, &result);
    byte_number += 12;


    if(check_number_mask(&blockette_pointer[byte_number], 15, "Number of coefficients", 3, 0, 0, 0, &result)) {
        number_of_coefficients = return_number(&blockette_pointer[byte_number], 3);
	byte_number += 3;

	if ((blockette_length - byte_number) % 24 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tField 16 and 17 repeat %i times, while they should repeat %i times", number_of_coefficients, (blockette_length - byte_number) / 24, number_of_coefficients);
	    add_error_to_result(&blockette_pointer[byte_number - 3], 15, "Number of coefficients", 3, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, field 16 and 17 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 3], 15, "Number of coefficients", 3, 0, error_string, &result);
	}
	else {
	    for(i=0; i < number_of_coefficients; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 16, "Polynomial coefficient", 12, 3, 9, 1, &result);
	        byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 17, "Polynomial coefficient error", 12, 3, 9, 1, &result);
	        byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of field 16 and 17 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 15, "Number of coefficients", 3, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_43(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_zeros;
    int num_poles;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 43;
    strncpy(result.blockette_name, "Response (Poles & Zeros) Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 043", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 50, -1, &result);
    if (blockette_length < 50) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Response type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'C', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'D', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, C, or D)");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Response type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 6, "Stage signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 7, "Stage signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    check_number_mask(&blockette_pointer[byte_number], 8, "AO normalization factor (1.0 if none)", 12, 3, 9, 1, &result);
    byte_number += 12;
    check_number_mask(&blockette_pointer[byte_number], 9, "Normalization frequency (Hz)", 12, 3, 9, 1, &result);
    byte_number += 12;


    if(check_number_mask(&blockette_pointer[byte_number], 10, "Number of complex zeros", 3, 0, 0, 0, &result)) {
        num_zeros = return_number(&blockette_pointer[byte_number], 3);
	byte_number += 3;

	for(i=0; i < num_zeros; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 11, "Real zero", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 12, "Imaginary zero", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 13, "Real zero error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 14, "Imaginary zero error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	}

	if(check_number_mask(&blockette_pointer[byte_number], 15, "Number of complex poles", 3, 0, 0, 0, &result)) {
	    num_poles = return_number(&blockette_pointer[byte_number], 3);
	    byte_number += 3;

	    if ((blockette_length - byte_number) % 48 != 0) {
	        sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 16 - 19 repeat %i times, while they should repeat %i times", num_poles, (blockette_length - byte_number) / 48, num_poles);
		add_error_to_result(&blockette_pointer[byte_number - 3], 15, "Number of complex poles", 3, 1, error_string, &result);

		sprintf(error_string, "Because of the above error, fields 16 - 19 were not checked in any way");
		add_error_to_result(&blockette_pointer[byte_number - 3], 15, "Number of complex poles", 3, 0, error_string, &result);
	    }
	    else {
	        for(i=0; i < num_poles; ++i) {
		    check_number_mask(&blockette_pointer[byte_number], 16, "Real pole", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 17, "Imaginary pole", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 18, "Real pole error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 19, "Imaginary pole error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		}
		
		if (byte_number != blockette_length) {
		    print_blockette_length_problem(byte_number, blockette_length, &result); 
		}
	    }
	}
	else {
	    sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	    add_error_to_result(&blockette_pointer[byte_number], 15, "Number of complex poles", 3, 0, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 10, "Number of complex zeros", 3, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_44(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_numerators;
    int num_denominators;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 44;
    strncpy(result.blockette_name, "Response (Coefficients) Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 044", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 28, -1, &result);
    if (blockette_length < 28) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Response type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'C', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Response type", 1, 'D', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, C, or D)");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Response type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 6, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 7, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;


    if(check_number_mask(&blockette_pointer[byte_number], 8, "Number of numerators", 4, 0, 0, 0, &result)) {
        num_numerators = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	for(i=0; i < num_numerators; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 9, "Numerator coefficient", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 10, "Numerator error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	}

	if(check_number_mask(&blockette_pointer[byte_number], 11, "Number of denominators", 4, 0, 0, 0, &result)) {
	    num_denominators = return_number(&blockette_pointer[byte_number], 4);
	    byte_number += 4;

	    if ((blockette_length - byte_number) % 24 != 0) {
	        sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 12 and 13 repeat %i times, while they should repeat %i times", num_denominators, (blockette_length - byte_number) / 24, num_denominators);
		add_error_to_result(&blockette_pointer[byte_number - 4], 11, "Number of denominators", 4, 1, error_string, &result);

		sprintf(error_string, "Because of the above error, fields 12 and 13 were not checked in any way");
		add_error_to_result(&blockette_pointer[byte_number - 4], 11, "Number of denominators", 4, 0, error_string, &result);
	    }
	    else {
	        for(i=0; i < num_denominators; ++i) {
		    check_number_mask(&blockette_pointer[byte_number], 12, "Denominator coefficient", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 13, "Denominator error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		}
		
		if (byte_number != blockette_length) {
		    print_blockette_length_problem(byte_number, blockette_length, &result); 
		}
	    }
	}
	else {
	    sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	    add_error_to_result(&blockette_pointer[byte_number], 11, "Number of denominators", 4, 0, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Number of numerators", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_45(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{

    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_responses;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 45;
    strncpy(result.blockette_name, "Response List Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 045", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 23, -1, &result);
    if (blockette_length < 23) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 5, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 6, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    if(check_number_mask(&blockette_pointer[byte_number], 7, "Number of responses listed", 4, 0, 0, 0, &result)) {
        num_responses = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 60 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 8 - 12 repeat %i times, while they should repeat %i times", num_responses, (blockette_length - byte_number) / 60, num_responses);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 7, "Number of responses listed", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 8 - 12 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 7, "Number of responses listed", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < num_responses; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 8, "Frequency (Hz)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 9, "Amplitude", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 10, "Amplitude error", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 11, "Phase angle (degrees)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 12, "Phase error (degrees)", 12, 3, 9, 1, &result);
		byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 8 - 12 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 7, "Number of responses listed", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_46(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_corners;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 46;
    strncpy(result.blockette_name, "Generic Response Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 046", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 23, -1, &result);
    if (blockette_length < 23) {
        sprintf(error_string, "Because the blockette is too short, it was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Response Lookup Key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 5, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 6, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    if(check_number_mask(&blockette_pointer[byte_number], 7, "Number of corners listed", 4, 0, 0, 0, &result)) {
        num_corners = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 24 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 8 and 9 repeat %i times, while they should repeat %i times", num_corners, (blockette_length - byte_number) / 24, num_corners);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 7, "Number of corners listed", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 8 and 9 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 7, "Number of cornders listed", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < num_corners; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 8, "Corner frequency (Hz)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 9, "Corner slope (db/decade)", 12, 3, 9, 1, &result);
		byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 8 and 9 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 7, "Number of corners listed", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_47(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 47;
    strncpy(result.blockette_name, "Decimation Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 047", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 55, 79, &result);
    if (blockette_length < 55) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    check_number_mask(&blockette_pointer[byte_number], 3, "Response lookup key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 5, "Input sample rate", 10, 2, 7, 0, &result);
    byte_number += 10;
    check_number_mask(&blockette_pointer[byte_number], 6, "Decimation factor", 5, 0, 0, 0, &result);
    byte_number += 5;
    check_number_mask(&blockette_pointer[byte_number], 7, "Decimation offset", 5, 0, 0, 0, &result);
    byte_number += 5;
    check_number_mask(&blockette_pointer[byte_number], 8, "Estimated delay (seconds)", 11, 3, 8, 1, &result);
    byte_number += 11;
    check_number_mask(&blockette_pointer[byte_number], 9, "Correction applied (seconds)", 11, 3, 8, 1, &result);
    byte_number += 11;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_48(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_history;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 48;
    strncpy(result.blockette_name, "Channel Sensitivity/Gain Dictionary Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 048", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 39, -1, &result);
    if (blockette_length < 39) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Response lookup key", 4, 0, 0, 0, &result);
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response name", 25, 1, "UNL_", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 5, "Sensitivity/gain", 12, 3, 9, 1, &result);
    byte_number += 12;
    check_number_mask(&blockette_pointer[byte_number], 6, "Frequency (Hz)", 12, 3, 9, 1, &result);
    byte_number += 12;

    if(check_number_mask(&blockette_pointer[byte_number], 7, "Number of history values", 2, 0, 0, 0, &result)) {
        num_history = return_number(&blockette_pointer[byte_number], 2);
	byte_number += 2;

	for(i=0; i < num_history; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 8, "Sensitivity for calibration", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 9, "Frequency of calibration sensitivity", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 10, "Time of above calculation", 22, &result);
	    byte_number += bytes_function_read;
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 8 - 10 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 7, "Number of history values", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


