#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "sanity.h"
#include "rdseed.h"


static int parsed_station = 0;

static char station[10];
static char channel[5];
static char network[5];
static char location[3];



/* this is the function that should be called when a blockette should
   be checked for errors. It takes four arguments. A pointer to the
   data in the blockette, the number of the record it starts in, the
   number of the record it ends in (the two may be different), and
   how verbose the output should be. (1 for the least, 3 for the most) */

int blockette_sanity_checker(blockette_pointer, start_record, end_record, verbose_option) 
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int error = 0;

    char blockette_number_char[4];

    blockette_number_char[3] = '\0';

    /* assume the first 3 characters in the blockette are correct.
       They should contain the blockette number. */

    strncpy(blockette_number_char, blockette_pointer, 3);
    blockette_number = atoi(blockette_number_char);

    /* this function is mostly just a big switch statement. Once
       the blockette number is found, enter into the appropriate 
       function.

       Each of the functions returns an error level. 
       See the comments at the beginning of sanity.h for an explanation */

    switch (blockette_number) {
    case 5:
        error = check_blockette_5(blockette_pointer, start_record, end_record, verbose_option);
        break;
    case 8:
        error = check_blockette_8(blockette_pointer, start_record, end_record, verbose_option);
        break;
    case 10:
        error = check_blockette_10(blockette_pointer, start_record, end_record, verbose_option);

        break;
    case 11:
        error = check_blockette_11(blockette_pointer, start_record, end_record, verbose_option);
        break;
    case 12:
        error = check_blockette_12(blockette_pointer, start_record, end_record, verbose_option);
        break;
    case 30:
        error = check_blockette_30(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 31:
        error = check_blockette_31(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 32:
        error = check_blockette_32(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 33:
        error = check_blockette_33(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 34:
        error = check_blockette_34(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 35:
        error = check_blockette_35(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 41:
        error = check_blockette_41(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 42:
        error = check_blockette_42(blockette_pointer, start_record, end_record,
verbose_option);
        break;
    case 43:
        error = check_blockette_43(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 44:
        error = check_blockette_44(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 45:
        error = check_blockette_45(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 46:
        error = check_blockette_46(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 47:
        error = check_blockette_47(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 48:
        error = check_blockette_48(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 50:
        error = check_blockette_50(blockette_pointer, start_record, end_record, verbose_option);

	/* Save station/network for error reporting later */
	strncpy(station, blockette_pointer+7, 5);
	station[5] = 0;

	if (type10.version >= 2.3)
	{
		/* network is always the last field */
		strcpy(network, 
			blockette_pointer + (strlen(blockette_pointer) - 2));
	
	}
	else
		strcpy(network, "N/A");

	parsed_station = 1;

	break;
    case 51:
        error = check_blockette_51(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 52:
        error = check_blockette_52(blockette_pointer, start_record, end_record, verbose_option);

	/* save the channel for later error reports */
	strncpy(channel, blockette_pointer + 9, 3);
	channel[3] = 0;

	/* ditto location */
	strncpy(location, blockette_pointer + 7, 2);
        location[2] = 0;

	break;
    case 53:
        error = check_blockette_53(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 54:
        error = check_blockette_54(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 55:
        error = check_blockette_55(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 56:
        error = check_blockette_56(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 57:
        error = check_blockette_57(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 58:
        error = check_blockette_58(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 59:
        error = check_blockette_59(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 60:
        error = check_blockette_60(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 61:
        error = check_blockette_61(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 62:
        error = check_blockette_62(blockette_pointer, start_record, end_record,
verbose_option);
	break;
    case 70:
        error = check_blockette_70(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 71:
        error = check_blockette_71(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 72:
        error = check_blockette_72(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 73:
        error = check_blockette_73(blockette_pointer, start_record, end_record, verbose_option);
	break;
    case 74:
        error = check_blockette_74(blockette_pointer, start_record, end_record, verbose_option);
	break;
    default:
        error = 2;
	fprintf(stderr, "WARNING: Blockette number %i not recognized\n", blockette_number);
	break;
    }

	if (error && parsed_station)
	{
		fprintf(stderr, 
			"Check network,station,channel,location: %s,%s,%s,%s\n\n",
				network, station, channel, location);
	}    

    return (error);
}
