#include "sanity.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

extern double version;
extern int relax_flag;



int check_blockette_50(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{

    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int time1_length;
    int time2_length;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 50;
    strncpy(result.blockette_name, "Station Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 050", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    if (version < 2.3) {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 62, 164, &result);
      if (blockette_length < 62) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }
    else {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 64, 166, &result);
      if (blockette_length < 64) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Station call letters", 5, 0, "UN", &result);
    byte_number += bytes_function_read;

    if(check_number_mask(&blockette_pointer[byte_number], 4, "Latitude (degrees)", 10, 4, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 4, "Latitude (degrees)", 10, (double)-90, (double)90, &result);
    }
    else {

        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 4, "Latitude (degrees)", 10, 0, error_string, &result);

    }
    byte_number += 10;

    if(check_number_mask(&blockette_pointer[byte_number], 5, "Longitude (degrees)", 11, 5, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 5, "Longitude (degrees)", 11, (double)-180, (double)180, &result);
    }
    else {

        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Longitude (degrees)", 11, 0, error_string, &result);

    }
    byte_number += 11;

    if (check_number_mask(&blockette_pointer[byte_number], 6, "Elevation (m)", 7, 6, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 6, "Elevation (m)", 7, -9999.9, 9999.9, &result);
    }
    else {

        sprintf(error_string, "Because of above error, the value of the Elevation could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Elevation (m)", 7, 0, error_string, &result);
    }
    byte_number += 7;

    check_number_mask(&blockette_pointer[byte_number], 7, 
			"Number of channels", 4, 0, 0, 0, &result);

    byte_number += 4;
    check_number_mask(&blockette_pointer[byte_number], 8, "Number of station comments", 3, 0, 0, 0, &result);
    byte_number += 3;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 9, "Site name", 60, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 10, "Network identifier code", 3, 0, 0, 0, &result);
    byte_number += 3;

    if (check_number_mask(&blockette_pointer[byte_number], 11, "32 bit word order", 4, 0, 0, 0, &result)) {
        check_number_one_of_two_values(&blockette_pointer[byte_number], 11, "32 bit word order", 4, (double)123, (double)3210, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 11, "32 bit word order", 4, 0, error_string, &result);
    }
    byte_number += 4;

    if (check_number_mask(&blockette_pointer[byte_number], 12, "16 bit word order", 2, 0, 0, 0, &result)) {
        check_number_one_of_two_values(&blockette_pointer[byte_number], 12, "16 bit word order", 2, (double)01, (double)10, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 12, "16 bit word order", 2, 0, error_string, &result);
    }
    byte_number += 2;


    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 13, "Start effective date", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 14, "End effective date", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - time1_length], 13, "Start effective date", time1_length, &blockette_pointer[byte_number - time2_length], 14, "End effective date", time2_length, &result);


    check_char_one_of_two_values(&blockette_pointer[byte_number], 15, "Update flag", 1, 'N', 'U', &result);
    ++byte_number;

    /* if (byte_number == blockette_length) {
        if (result.error) {
	    print_errors(&result, verbose_option);
	}

	return(result.error);
    }*/

    /*    if (byte_number < blockette_length) {*/
    if(version >= 2.3) {
        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 16, "Network Code", 2, 0, "UN", &result);
	byte_number += bytes_function_read;
    }

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }


    return(result.error);
}


int check_blockette_51(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];

    int time1_length;
    int time2_length;

    struct result_struct result;

    result.blockette_number = 51;
    strncpy(result.blockette_name, "Station Comment Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 051", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 21, 63, &result);
    if (blockette_length < 21) {
        sprintf(error_string, "Because the blockette is too short, it was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 3, "Beginning effective time", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "End effective time", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - time1_length], 3, "Beginning effective time", time1_length, &blockette_pointer[byte_number - time2_length], 4, "End effective time", time2_length, &result);

    check_number_mask(&blockette_pointer[byte_number], 5, "Comment code key", 4, 0, 0, 0, &result);
    byte_number += 4;
    check_number_mask(&blockette_pointer[byte_number], 6, "Comment level", 6, 0, 0, 0, &result);
    byte_number += 6;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_52(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int time1_length;
    int time2_length;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 52;
    strncpy(result.blockette_name, "Channel Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 052", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 104, 203, &result);
    if (blockette_length < 104) {
        sprintf(error_string, "Because the blockette is too short, it was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Location identifier", 2, 0, "UN", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Channel identifier", 3, 0, "UN", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 5, "Subchannel identifier", 4, 0, 0, 0, &result);
    byte_number += 4;
    check_number_mask(&blockette_pointer[byte_number], 6, "Instrument identifier", 3, 0, 0, 0, &result);
    byte_number += 3;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 7, "Optional comment", 30, 1, "UNLPS", &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 8, "Units of signal response", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 9, "Units of calibration input", 3, 0, 0, 0, &result);
    byte_number += 3;


    if(check_number_mask(&blockette_pointer[byte_number], 10, "Latitude (degrees)", 10, 4, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 10, "Latitude (degrees)", 10, (double)-90, (double)90, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 10, "Latitude (degrees)", 10, 0, error_string, &result);
    }
    byte_number += 10;

    if(check_number_mask(&blockette_pointer[byte_number], 11, "Longitude (degrees)", 11, 5, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 11, "Longitude (degrees)", 11, (double)-180, (double)180, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 11, "Longitude (degrees)", 11, 0, error_string, &result);
    }
    byte_number += 11;

    if (check_number_mask(&blockette_pointer[byte_number], 12, "Elevation (m)", 7, 6, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 12, "Elevation (m)", 7, (double)-20000, (double)10000, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the Elevation could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 12, "Elevation (m)", 7, 0, error_string, &result);
    }
    byte_number += 7;

    check_number_mask(&blockette_pointer[byte_number], 13, "Local depth (m)", 5, 4, 0, 1, &result);  
    byte_number += 5;

    if(check_number_mask(&blockette_pointer[byte_number], 14, "Azimuth (degrees)", 5, 4, 0, 0, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 14, "Azimuth (degrees)", 5, (double)0, (double)360, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 14, "Azimuth (degrees)", 5, 0, error_string, &result);
    }
    byte_number += 5;

    if(check_number_mask(&blockette_pointer[byte_number], 15, "Dip (degrees)", 5, 4, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 15, "Dip (degrees)", 5, (double)-90, (double)90, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 15, "Dip (degrees)", 5, 0, error_string, &result);
    }
    byte_number += 5;

    check_number_mask(&blockette_pointer[byte_number], 16, "Data format identifier code", 4, 0, 0, 0, &result);  
    byte_number += 4;

    if(check_number_mask(&blockette_pointer[byte_number], 17, "Data record length", 2, 0, 0, 0, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 17, "Data record length", 2, (double)8, (double)12, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 17, "Data record length", 2, 0, error_string, &result);
    }
    byte_number += 2;

    /* workaround, could cause problems if the field actually does have a problem. This
       was done because every test case had this problem. Because seed readers will just do
       an atof on this field, the spaces will not really matter 
       What was happening was people were putting a space in front of the field. While this is
       legal, they would then move the decimal point to the right one character position, making
       their data illegal */
    /*if (blockette_pointer[byte_number] == ' ') {
        for (i=0; i < 5; ++i) {
	    blockette_pointer[byte_number + i] = blockette_pointer[byte_number + i + 1];
	}
	blockette_pointer[byte_number + i] = '0';
    }*/

    	check_number_mask(&blockette_pointer[byte_number], 
					18, "Sample Rate (Hz)", 
					10, 2, 7, 0, &result);  


    byte_number += 10;

    /* workaround, could cause problems if the field actually does have a problem */
    /* see above for comments on why this was done */
    /*if (blockette_pointer[byte_number] == ' ') {
        for (i=0; i < 5; ++i) {
	    blockette_pointer[byte_number + i] = blockette_pointer[byte_number + i + 1];
	}
	blockette_pointer[byte_number + i] = '0';
    }*/
    check_number_mask(&blockette_pointer[byte_number], 19, "Max clock drift", 10, 2, 7, 0, &result);  
    byte_number += 10;

    check_number_mask(&blockette_pointer[byte_number], 20, "Number of comments", 4, 0, 0, 0, &result);  
    byte_number += 4;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 21, "Channel flags", 26, 1, "U", &result);

    for (i=0; i < bytes_function_read - 1; ++i) {
        if (!check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'T', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'C', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'H', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'G', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'W', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'F', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'S', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'I', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'E', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'M', 0, &result)
	    && !check_char_value(&blockette_pointer[byte_number + i], 21, "Channel flags", bytes_function_read, 'B', 0, &result)) {

	    sprintf(error_string, "The character in the field is not one of the \n\t\taccepted values (T, C, H, G, W, F, S, I, E, M, or B)");
	    add_error_to_result(&blockette_pointer[byte_number], 21, "Channel flags", bytes_function_read, 0, error_string, &result);
	}
    }
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 22, "Start date", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 23, "End date", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - time1_length], 22, "Start date", time1_length, &blockette_pointer[byte_number - time2_length], 23, "End date", time2_length, &result);


    check_char_one_of_two_values(&blockette_pointer[byte_number], 24, "Update flag", 1, 'N', 'U', &result);
    ++byte_number;


    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_53(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_zeros;
    int num_poles;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 53;
    strncpy(result.blockette_name, "Response (Poles & Zeros) Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 053", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 46, -1, &result);
    if (blockette_length < 46) {
        sprintf(error_string, "Because the blockette is too short, it was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 'C', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 'D', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, C, or D)");
	add_error_to_result(&blockette_pointer[byte_number], 3, "Transfer function type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 4, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;
    check_number_mask(&blockette_pointer[byte_number], 5, "Stage signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 6, "Stage signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    check_number_mask(&blockette_pointer[byte_number], 7, "AO normalization factor (1.0 if none)", 12, 3, 9, 1, &result);
    byte_number += 12;
    check_number_mask(&blockette_pointer[byte_number], 8, "Normalization frequency (Hz)", 12, 3, 9, 1, &result);
    byte_number += 12;


    if(check_number_mask(&blockette_pointer[byte_number], 9, "Number of complex zeros", 3, 0, 0, 0, &result)) {
        num_zeros = return_number(&blockette_pointer[byte_number], 3);
	byte_number += 3;

	for(i=0; i < num_zeros; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 10, "Real zero", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 11, "Imaginary zero", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 12, "Real zero error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 13, "Imaginary zero error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	}

	if(check_number_mask(&blockette_pointer[byte_number], 14, "Number of complex poles", 3, 0, 0, 0, &result)) {
	    num_poles = return_number(&blockette_pointer[byte_number], 3);
	    byte_number += 3;

	    if ((blockette_length - byte_number) % 48 != 0) {
	        sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 15 - 18 repeat %i times, while they should repeat %i times", num_poles, (blockette_length - byte_number) / 48, num_poles);
		add_error_to_result(&blockette_pointer[byte_number - 3], 14, "Number of complex poles", 3, 1, error_string, &result);

		sprintf(error_string, "Because of the above error, fields 15 - 18 were not checked in any way");
		add_error_to_result(&blockette_pointer[byte_number - 3], 14, "Number of complex poles", 3, 0, error_string, &result);
	    }
	    else {
	        for(i=0; i < num_poles; ++i) {
		    check_number_mask(&blockette_pointer[byte_number], 15, "Real pole", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 16, "Imaginary pole", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 17, "Real pole error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 18, "Imaginary pole error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		}
		
		if (byte_number != blockette_length) {
		    print_blockette_length_problem(byte_number, blockette_length, &result); 
		}
	    }
	}
	else {
	    sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	    add_error_to_result(&blockette_pointer[byte_number], 14, "Number of complex poles", 3, 0, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 9, "Number of complex zeros", 3, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}

int check_blockette_54(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{

int flag = 0;

    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_numerators;
    int num_denominators;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 54;
    strncpy(result.blockette_name, "Response (Coefficients) Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 054", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 24, -1, &result);
    if (blockette_length < 24) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Response type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 3, "Response type", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Response type", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Response type", 1, 'C', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 3, "Response type", 1, 'D', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, C, or D)");
	add_error_to_result(&blockette_pointer[byte_number], 3, "Response type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 4, "Signal sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;
    check_number_mask(&blockette_pointer[byte_number], 5, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 6, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;


    if(check_number_mask(&blockette_pointer[byte_number], 7, "Number of numerators", 4, 0, 0, 0, &result)) {
        num_numerators = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	for(i=0; i < num_numerators; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 8, "Numerator coefficient", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 9, "Numerator error", 12, 3, 9, 1, &result);
	    byte_number += 12;
	}

	if(check_number_mask(&blockette_pointer[byte_number], 10, "Number of denominators", 4, 0, 0, 0, &result)) {
	    num_denominators = return_number(&blockette_pointer[byte_number], 4);
	    byte_number += 4;

	    if ((blockette_length - byte_number) % 24 != 0) {
	        sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 11 and 12 repeat %i times, while they should repeat %i times", num_denominators, (blockette_length - byte_number) / 24, num_denominators);
		add_error_to_result(&blockette_pointer[byte_number - 4], 10, "Number of denominators", 4, 1, error_string, &result);

		sprintf(error_string, "Because of the above error, fields 11 and 12 were not checked in any way");
		add_error_to_result(&blockette_pointer[byte_number - 4], 10, "Number of denominators", 4, 0, error_string, &result);
	    }
	    else {
	        for(i=0; i < num_denominators; ++i) {
if (flag)
{
printf("I = %d\n", i);

}

		    check_number_mask(&blockette_pointer[byte_number], 11, "Denominator coefficient", 12, 3, 9, 1, &result);
		    byte_number += 12;
		    check_number_mask(&blockette_pointer[byte_number], 12, "Denominator error", 12, 3, 9, 1, &result);
		    byte_number += 12;
		}
		
		if (byte_number != blockette_length) {
		    print_blockette_length_problem(byte_number, blockette_length, &result); 
		}
	    }
	}
	else {
	    sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	    add_error_to_result(&blockette_pointer[byte_number], 10, "Number of denominators", 4, 0, error_string, &result);
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 7, "Number of numerators", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_55(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_responses;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 55;
    strncpy(result.blockette_name, "Response List Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 055", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 19, -1, &result);
    if (blockette_length < 19) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;

    check_number_mask(&blockette_pointer[byte_number], 4, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 5, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    if(check_number_mask(&blockette_pointer[byte_number], 6, "Number of responses listed", 4, 0, 0, 0, &result)) {
        num_responses = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 60 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 7 - 11 repeat %i times, while they should repeat %i times", num_responses, (blockette_length - byte_number) / 60, num_responses);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 6, "Number of responses listed", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 7 - 11 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 6, "Number of responses listed", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < num_responses; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 7, "Frequency (Hz)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 8, "Amplitude", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 9, "Amplitude error", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 10, "Phase angle (degrees)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 11, "Phase error (degrees)", 12, 3, 9, 1, &result);
		byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 7 - 11 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Number of responses listed", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_56(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_corners;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 56;
    strncpy(result.blockette_name, "Generic Response Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 056", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 19, -1, &result);
    if (blockette_length < 19) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;
    check_number_mask(&blockette_pointer[byte_number], 4, "Signal input units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 5, "Signal output units", 3, 0, 0, 0, &result);
    byte_number += 3;

    if(check_number_mask(&blockette_pointer[byte_number], 6, "Number of corners listed", 4, 0, 0, 0, &result)) {
        num_corners = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 24 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tFields 7 and 8 repeat %i times, while they should repeat %i times", num_corners, (blockette_length - byte_number) / 24, num_corners);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 6, "Number of corners listed", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 7 and 8 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 6, "Number of cornders listed", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < num_corners; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 7, "Corner frequency (Hz)", 12, 3, 9, 1, &result);
		byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 8, "Corner slope (db/decade)", 12, 3, 9, 1, &result);
		byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 7 and 8 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Number of corners listed", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_57(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];

    int i;

    struct result_struct result;

    result.blockette_number = 57;
    strncpy(result.blockette_name, "Decimation Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 057", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 51, 51, &result);
    if (blockette_length < 51) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;

    /* workaround, could cause problems if the field actually does have a problem */
    /* see comments in check_blockette_52 code for why this was done */
    /* if (blockette_pointer[byte_number] == ' ') {
        for (i=0; i < 5; ++i) {
	    blockette_pointer[byte_number + i] = blockette_pointer[byte_number + i + 1];
	}
	blockette_pointer[byte_number + i] = '0';
    }*/

    	check_number_mask(&blockette_pointer[byte_number], 
				  4, "Input sample rate (Hz)", 
				  10, 2, 
				  7, 0, 
				  &result);

    byte_number += 10;

    check_number_mask(&blockette_pointer[byte_number], 5, "Decimation factor", 5, 0, 0, 0, &result);
    byte_number += 5;
    check_number_mask(&blockette_pointer[byte_number], 6, "Decimation offset", 5, 0, 0, 0, &result);
    byte_number += 5;
    check_number_mask(&blockette_pointer[byte_number], 7, "Estimated delay (seconds)", 11, 3, 8, 1, &result);
    byte_number += 11;
    check_number_mask(&blockette_pointer[byte_number], 8, "Correction applied (seconds)", 11, 3, 8, 1, &result);
    byte_number += 11;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_58(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int stage_number;
    int num_history;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 58;
    strncpy(result.blockette_name, "Channel Sensitivity/Gain Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 058", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 35, -1, &result);
    if (blockette_length < 35) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Stage sequence number", 2, 0, 0, 0, &result);
    stage_number = return_number(&blockette_pointer[byte_number], 2);
    byte_number += 2;

    check_number_mask(&blockette_pointer[byte_number], 4, "Sensitivity/gain", 12, 3, 9, 1, &result);
    byte_number += 12;
    check_number_mask(&blockette_pointer[byte_number], 5, "Frequency (Hz)", 12, 3, 9, 1, &result);
    byte_number += 12;

    if(check_number_mask(&blockette_pointer[byte_number], 6, "Number of history values", 2, 0, 0, 0, &result)) {
        num_history = return_number(&blockette_pointer[byte_number], 2);
	byte_number += 2;

	if (stage_number != 0 && num_history != 0) {
	    sprintf(error_string, "Stage number is not equal to zero, implying that this blockette contains\n\t\tgain values and that there will be no history, however, there is a history");
	    add_error_to_result(&blockette_pointer[byte_number], 6, "Number of history values", 2, 0, error_string, &result);
	}

	for(i=0; i < num_history; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 7, "Sensitivity for calibration", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    check_number_mask(&blockette_pointer[byte_number], 8, "Frequency of calibration sensitivity", 12, 3, 9, 1, &result);
	    byte_number += 12;
	    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 10, "Time of above calculation", 22, &result);
	    byte_number += bytes_function_read;
	}

	if ((byte_number != blockette_length) && (!relax_flag)) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of fields 7 - 9 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Number of history values", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

	if ((byte_number != blockette_length) && (!relax_flag))
		return 1;	/* flag error */
    	else
		return(result.error);
}


int check_blockette_59(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];

    int time1_length;
    int time2_length;

    struct result_struct result;

    result.blockette_number = 59;
    strncpy(result.blockette_name, "Channel Comment Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 059", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 20, 63, &result);
    if (blockette_length < 20) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 3, "Beginning effective time", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "End effective time", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - time1_length], 3, "Beginning effective time", time1_length, &blockette_pointer[byte_number - time2_length], 4, "End effective time", time2_length, &result);

    check_number_mask(&blockette_pointer[byte_number], 5, "Comment code key", 4, 0, 0, 0, &result);
    byte_number += 4;
    check_number_mask(&blockette_pointer[byte_number], 6, "Comment level", 6, 0, 0, 0, &result);
    byte_number += 6;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_60(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;

    int i, j;
    int error = 0;
    int num_stages;
    int num_responses;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 60;
    strncpy(result.blockette_name, "Response Reference Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 060", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 13, -1, &result);
    if (blockette_length < 13) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if(check_number_mask(&blockette_pointer[byte_number], 3, "Number of stages", 2, 0, 0, 0, &result)) {
        num_stages = return_number(&blockette_pointer[byte_number], 2);
	byte_number += 2;

	for(i=0; i < num_stages; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 4, "Stage sequence number", 2, 0, 0, 0, &result);
	    byte_number += 2;

	    if (check_number_mask(&blockette_pointer[byte_number], 5, "Number of responses", 2, 0, 0, 0, &result)) {
	        num_responses = return_number(&blockette_pointer[byte_number], 2);
		byte_number += 2;

		for(j=0; j < num_responses; ++j) {
		    check_number_mask(&blockette_pointer[byte_number], 6, "Response lookup key", 4, 0, 0, 0, &result);
		    byte_number += 4;
		}
	    }
	    else {
	        sprintf(error_string, "Because the value in the field in the %i stage did not conform to the mask,\n\t\tthe rest of the blockette could not be checked", i);
		add_error_to_result(&blockette_pointer[byte_number], 5, "Number of responses", 2, 0, error_string, &result);
		error = 1;
		break;
	    }
	}

	if (byte_number != blockette_length && !error) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe rest of the blocktte could be not checked");
	add_error_to_result(&blockette_pointer[byte_number], 3, "Number of stages", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_61(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int number_of_coefficients;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 61;
    strncpy(result.blockette_name, "FIR Response Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 061", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 21, -1, &result);
    if (blockette_length < 21) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    check_number_mask(&blockette_pointer[byte_number], 3, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;
	
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Response Name", 25, 1, "UNLPS_", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'A', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'B', 0, &result)
	&& !check_char_value(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 'C', 0, &result)) {

        sprintf(error_string, "The character in the field is not one of the accepted values (A, B, or C)");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Symmetry Code", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 6, "Signal In Units", 3, 0, 0, 0, &result);
    byte_number += 3;
    check_number_mask(&blockette_pointer[byte_number], 7, "Signal Out Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    if(check_number_mask(&blockette_pointer[byte_number], 8, "Number of Coefficients", 4, 0, 0, 0, &result)) {
        number_of_coefficients = return_number(&blockette_pointer[byte_number], 4);
	byte_number += 4;

	if ((blockette_length - byte_number) % 14 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tField 9 repeats %i times, while it should repeat %i times", number_of_coefficients, (blockette_length - byte_number) / 14, number_of_coefficients);
	    add_error_to_result(&blockette_pointer[byte_number - 4], 8, "Number of Factors", 4, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, field 9 was not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 4], 8, "Number of Factors", 4, 0, error_string, &result);
	}
	else {
	    for(i=0; i < number_of_coefficients; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 9, "FIR Coefficient", 14, 3, 11, 1, &result);
		byte_number += 14;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of field 9 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Number of Coefficients", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_62(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int number_of_coefficients;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 62;
    strncpy(result.blockette_name, "Polynomial Response Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 062", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 21, -1, &result);
    if (blockette_length < 21) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Transfer Function Type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 3, "Transfer Function Type", 1, 'P', 0, &result))    {

        sprintf(error_string, "The character in the field is not one of the accepted value (P)");
	add_error_to_result(&blockette_pointer[byte_number], 3, "Transfer Function Type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 4, "Stage sequence number", 2, 0, 0, 0, &result);
    byte_number += 2;


    check_number_mask(&blockette_pointer[byte_number], 5, "Signal In Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    check_number_mask(&blockette_pointer[byte_number], 6, "Signal Out Units", 3, 0, 0, 0, &result);
    byte_number += 3;


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 7, "Polynomial Approximation Type", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 7, "Polynomial Approximation Type", 1, 'M', 0, &result))    {

        sprintf(error_string, "The character in the field is not one of the accepted value (M)");
	add_error_to_result(&blockette_pointer[byte_number], 7, "Polynomial Approximation Type", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 8, "Valid Frequency Units", 1, 0, "U", &result);

    if (!check_char_value(&blockette_pointer[byte_number], 8, "Valid Frequency Units", 1, 'A', 0, &result)
	&& (!check_char_value(&blockette_pointer[byte_number], 8, "Valid Frequency Units", 1, 'B', 0, &result))) {
        sprintf(error_string, "The character in the field is not one of the accepted value (A or B)");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Valid Frequency Units", 1, 1, error_string, &result);
    }
    byte_number += bytes_function_read;


    check_number_mask(&blockette_pointer[byte_number], 9, "Lower Valid Frequency Bound", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 10, "Upper Valid Frequency Bound", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 11, "Lower bound of approximation", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 12, "Upper bound of approximation", 12, 3, 9, 1, &result);
    byte_number += 12;


    check_number_mask(&blockette_pointer[byte_number], 13, "Maximum Absolute Error", 12, 3, 9, 1, &result);
    byte_number += 12;


    if(check_number_mask(&blockette_pointer[byte_number], 14, "Number of coefficients", 3, 0, 0, 0, &result)) {
        number_of_coefficients = return_number(&blockette_pointer[byte_number], 3);
	byte_number += 3;

	if ((blockette_length - byte_number) % 24 != 0) {
	    sprintf(error_string, "The number of components given in the field (%i) is wrong.\n\t\tField 15 and 16 repeat %i times, while they should repeat %i times", number_of_coefficients, (blockette_length - byte_number) / 24, number_of_coefficients);
	    add_error_to_result(&blockette_pointer[byte_number - 3], 14, "Number of coefficients", 3, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, field 15 and 16 were not checked in any way");
	    add_error_to_result(&blockette_pointer[byte_number - 3], 14, "Number of coefficients", 3, 0, error_string, &result);
	}
	else {
	    for(i=0; i < number_of_coefficients; ++i) {
	        check_number_mask(&blockette_pointer[byte_number], 15, "Polynomial coefficient", 12, 3, 9, 1, &result);
	        byte_number += 12;
	        check_number_mask(&blockette_pointer[byte_number], 16, "Polynomial coefficient error", 12, 3, 9, 1, &result);
	        byte_number += 12;
	    }

	    if (byte_number != blockette_length) {
	        print_blockette_length_problem(byte_number, blockette_length, &result); 
	    }
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe contents of field 15 and 16 were not checked");
	add_error_to_result(&blockette_pointer[byte_number], 14, "Number of coefficients", 3, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}
