#include "sanity.h"
#include <stdio.h>
#include <stdlib.h>


extern double version;
/*double version = (double)VERSION;*/


int check_blockette_70(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    
    int time1_length;
    int time2_length;

    struct result_struct result;

    char error_string[256];

    result.blockette_number = 70;
    strncpy(result.blockette_name, "Time Span Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 070", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 12, 54, &result);
    if (blockette_length < 12) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Time span flag", 1, 0, "U", &result);
    check_char_one_of_two_values(&blockette_pointer[byte_number], 3, "Time span flag", 1, 'E', 'P', &result);
    ++byte_number;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "Beginning time of data span", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 5, "End time of data span", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - time1_length], 4, "Begninning time of data span", time1_length, &blockette_pointer[byte_number - time2_length], 5, "End time of data span", time2_length, &result);


    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_71(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int number_of_magnitudes;
    char error_string[256];

    struct result_struct result;
	
    result.blockette_number = 71;
    strncpy(result.blockette_name, "Hypocenter Information Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 071", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    if (version < 2.3) {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 41, -1, &result);
      if (blockette_length < 41) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }
    else {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 50, -1, &result);
      if (blockette_length < 50) {
        sprintf(error_string, "Because the blockette is too short, it was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }


    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 3, "Origin time of event", 22, &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 4, "Hypocenter source identifier", 2, 0, 0, 0, &result);
    byte_number += 2;

    if(check_number_mask(&blockette_pointer[byte_number], 5, "Latitude of event (degrees)", 10, 4, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 5, "Latitude of event (degrees)", 10, (double)-90, (double)90, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 5, "Latitude of event (degrees)", 10, 0, error_string, &result);
    }
    byte_number += 10;

    if(check_number_mask(&blockette_pointer[byte_number], 6, "Longitude of event (degrees)", 11, 5, 0, 1, &result)) {
        check_number_between_two_values(&blockette_pointer[byte_number], 6, "Longitude (degrees)", 11, (double)-180, (double)180, &result);
    }
    else {
        sprintf(error_string, "Because of above error, the value of the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 6, "Longitude of event (degrees)", 11, 0, error_string, &result);
    }
    byte_number += 11;

    check_number_mask(&blockette_pointer[byte_number], 7, "Depth (m)", 7, 5, 0, 0, &result);  
    byte_number += 7;

    if (check_number_mask(&blockette_pointer[byte_number], 8, "Number of magnitudes", 2, 0, 0, 0, &result)) {
        number_of_magnitudes = return_number(&blockette_pointer[byte_number], 2);
        byte_number += 2;

	for(i=0; i < number_of_magnitudes; ++i) {
	    check_number_mask(&blockette_pointer[byte_number], 9, "Magnitude", 5, 3, 0, 0, &result);
	    byte_number += 5;
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 10, "Magnitude type", 10, 1, "UNLPS", &result);
	    byte_number += bytes_function_read;
	    check_number_mask(&blockette_pointer[byte_number], 11, "Magnitude source", 2, 0, 0, 0, &result);
	    byte_number += 2;
	}

	/*if (byte_number < blockette_length) {*/
	if (version >= 2.3) {
	    check_number_mask(&blockette_pointer[byte_number], 12, "Seismic region", 3, 0, 0, 0, &result);
	    check_number_between_two_values(&blockette_pointer[byte_number], 12, "Seismic region", 3, (double)1, (double)50, &result);
	    byte_number += 3;
	    check_number_mask(&blockette_pointer[byte_number], 13, "Seismic Location", 4, 0, 0, 0, &result);
	    check_number_between_two_values(&blockette_pointer[byte_number], 13, "Seismic Location", 4, (double)1, (double)729, &result);
	    byte_number += 4;
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 14, "Region Name", 40, 1, "UNLPS", &result);
	    byte_number += bytes_function_read;
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because of above error, the rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Number of magnitudes", 2, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }


    return(result.error);
}


int check_blockette_72(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;
    char error_string[256];


    struct result_struct result;

    result.blockette_number = 72;
    strncpy(result.blockette_name, "Event Phases Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 072", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    if (version < 2.3) {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 51, 91, &result);
      if (blockette_length < 51) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }
    else {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 55, 95, &result);
      if (blockette_length < 55) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }


    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Station identifier", 5, 0, "UN", &result);
    byte_number += bytes_function_read;
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Location identifier", 2, 0, "UN", &result);
    byte_number += bytes_function_read;
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Channel identifier", 3, 0, "UN", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 6, "Arrival time of phase", 22, &result);
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 7, "Amplitude of signal", 10, 2, 7, 0, &result);
    byte_number += 10;
    check_number_mask(&blockette_pointer[byte_number], 8, "Period of signal (seconds)", 10, 2, 7, 0, &result);
    byte_number += 10;
    check_number_mask(&blockette_pointer[byte_number], 9, "Signal-to-noise ratio", 10, 2, 7, 0, &result);
    byte_number += 10;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 10, "Name of phase", 20, 1, "UNLP", &result);
    byte_number += bytes_function_read;
 
    /*if (byte_number < blockette_length) {*/
    if (version >= 2.3) {
        check_number_mask(&blockette_pointer[byte_number], 11, "Source", 2, 0, 0, 0, &result);
	byte_number += 2;
	bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 12, "Network Code", 2, 0, "UN", &result);
	byte_number += bytes_function_read;
    }

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }
   
    return(result.error);
}


int check_blockette_73(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int num_data_pieces;
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 73;
    strncpy(result.blockette_name, "Time Span Data Start Index Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 073", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 11, -1, &result);
    if (blockette_length < 11) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[byte_number], 3, "Number of data pieces", 4, 0, 0, 0, &result)) {
        num_data_pieces = return_number(&blockette_pointer[byte_number], 4);
        byte_number += 4;

	for(i=0; i < num_data_pieces; ++i) {
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Station identifier of data piece", 5, 0, "UN", &result);
	    byte_number += bytes_function_read;
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Location identifier", 2, 0, "UN", &result);
	    byte_number += bytes_function_read;
	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 6, "Channel identifier", 3, 0, "UN", &result);
	    byte_number += bytes_function_read;
	    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 7, "Time of record", 22, &result);
	    byte_number += bytes_function_read;
	    check_number_mask(&blockette_pointer[byte_number], 8, "Sequence number of first record", 6, 0, 0, 0, &result);
	    byte_number += 6;
	    if (check_number_mask(&blockette_pointer[byte_number], 9, "Sub-sequence number", 2, 0, 0, 0, &result)) {
	        /*if (return_number(&blockette_pointer[byte_number], 2) == 0) {
		    sprintf(error_string, "The Sub-sequence number cannot equal zero");
		    add_error_to_result(&blockette_pointer[byte_number], 9, "Sub-sequence number", 2, 1, error_string, &result);
		}*/
	    }
	    else {
	        sprintf(error_string, "Because of above error, the number in the field could not be checked");
		add_error_to_result(&blockette_pointer[byte_number], 9, "Sub-sequence number", 2, 0, error_string, &result);
	    }
	    byte_number += 2;
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because of above error, the rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 3, "Number of data pieces", 4, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }


    return(result.error);
}


int check_blockette_74(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    int i;
    int time1_length;
    int time2_length;
    int num_accelerator_repeats;
    char error_string[256];

    struct result_struct result;

	char station[6];
	char network[3];
	char channel[4];
	char location[3];

	strcpy(network, "");
	strcpy(station, "");
	strcpy(channel, "");
	strcpy(location, "");

    result.blockette_number = 74;
    strncpy(result.blockette_name, "Time Series Index Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 074", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    if (version < 2.3) {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 40, -1, &result);
      if (blockette_length < 40) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }
    else {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 42, -1, &result);
      if (blockette_length < 42) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }


	/* save for later processing if error */
	strncpy(station, &blockette_pointer[byte_number], 5);
	station[5] = 0;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 3, "Station identifier", 5, 0, "UN", &result);

    byte_number += bytes_function_read;

	/* save for later processing if error */
        strncpy(location, &blockette_pointer[byte_number], 2);
        location[2] = 0;  

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Location identifier", 2, 0, "UN", &result);

    byte_number += bytes_function_read;

	strncpy(channel, &blockette_pointer[byte_number], 3); 
        channel[3] = 0; 

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Channel identifier", 3, 0, "UN", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 6, "Series start time", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_number_mask(&blockette_pointer[byte_number], 7, "Sequence number of first data", 6, 0, 0, 0, &result);
    byte_number += 6;

    if (check_number_mask(&blockette_pointer[byte_number], 8, "Sub-sequence number", 2, 0, 0, 0, &result)) {
        /*if (return_number(&blockette_pointer[byte_number], 2) == 0) {
	    sprintf(error_string, "The Sub-sequence number cannot equal zero");
	    add_error_to_result(&blockette_pointer[byte_number], 8, "Sub-sequence number", 2, 1, error_string, &result);
	}*/
    }
    else {
        sprintf(error_string, "Because of above error, the number in the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 8, "Sub-sequence number", 2, 0, error_string, &result);
    }
    byte_number += 2;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 9, "Series end time", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[byte_number - time2_length - 2 - 6 - time1_length], 6, "Series start time", time1_length, &blockette_pointer[byte_number - time2_length], 9, "Series end time", time2_length, &result);

    check_number_mask(&blockette_pointer[byte_number], 10, "Sequence number of last record", 6, 0, 0, 0, &result);
    byte_number += 6;

    if (check_number_mask(&blockette_pointer[byte_number], 11, "Sub-sequence number", 2, 0, 0, 0, &result)) {
        /*if (return_number(&blockette_pointer[byte_number], 2) == 0) {
	    sprintf(error_string, "The Sub-sequence number cannot equal zero");
	    add_error_to_result(&blockette_pointer[byte_number], 11, "Sub-sequence number", 2, 1, error_string, &result);
	}*/
    }
    else {
        sprintf(error_string, "Because of above error, the number in the field could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 11, "Sub-sequence number", 2, 0, error_string, &result);
    }
    byte_number += 2;

    if (check_number_mask(&blockette_pointer[byte_number], 12, "Number of accelerator repeats", 3, 0, 0, 0, &result)) {
        num_accelerator_repeats = return_number(&blockette_pointer[byte_number], 3);
        byte_number += 3;
	for(i=0; i < num_accelerator_repeats; ++i) {
	    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 13, "Record start time", 22, &result);
	    byte_number += bytes_function_read;
	    check_number_mask(&blockette_pointer[byte_number], 14, "Sequence number of record", 6, 0, 0, 0, &result);
	    byte_number += 6;
	    if (check_number_mask(&blockette_pointer[byte_number], 15, "Sub-sequence number", 2, 0, 0, 0, &result)) {
	        /*if (return_number(&blockette_pointer[byte_number], 2) == 0) {
		    sprintf(error_string, "The Sub-sequence number cannot equal zero");
		    add_error_to_result(&blockette_pointer[byte_number], 15, "Sub-sequence number", 2, 1, error_string, &result);
		}*/
	    }
	    else {
	        sprintf(error_string, "Because of above error, the number in the field could not be checked");
		add_error_to_result(&blockette_pointer[byte_number], 15, "Sub-sequence number", 2, 0, error_string, &result);
	    }
	    byte_number += 2;
	}

	/*if (byte_number < blockette_length) {*/
	if (version >= 2.3) {

		strncpy(network, &blockette_pointer[byte_number], 2); 
        	network[2] = 0; 

	    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 16, "Network Code", 2, 0, "UN", &result);
	    byte_number += bytes_function_read;
	}
	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because of above error, the rest of the blockette could not be checked");
	add_error_to_result(&blockette_pointer[byte_number], 12, "Number of accelerator repeats", 3, 0, error_string, &result);
    }

    if (result.error) {
        print_errors(&result, verbose_option);

	fprintf(stderr, "Check network,station,channel,location: %s,%s,%s,%s\n",
				network, station, channel, location);
    }


    return(result.error);
}
