#include "sanity.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*extern double version;*/
double version = (double)VERSION;


int check_blockette_5(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read;

    char temp_version[5];

    char error_string[256];

    struct result_struct result;

    /* initialiaze the result struct */
    result.blockette_number = 5;
    strncpy(result.blockette_name, "Field Volume Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 005", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    
    byte_number = 7;
        
    check_length(&blockette_pointer[3], 2, "Length of blockette", 4, blockette_length, 15, 36, &result);
    if (blockette_length < 15) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[7], 3, "Version of format", 4, 3, 0, 0, &result)) {
      if(check_number_between_two_values(&blockette_pointer[7], 3, "Version of format", 4, (double)0,(double)VERSION, &result) == 0) {
	strncpy(temp_version, &blockette_pointer[7], 4);
	temp_version[4] = '\0';
        version = atof(temp_version);
      }
      else {
        sprintf(error_string, "Because the value in the field was out of bounds,\n\t\tthe seed file is assumed to be version %f", (double) VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
      }
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked.\n\t\tThe version of the seed file is also assumed to be %f", (double)VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
    }

    if (check_number_mask(&blockette_pointer[11], 4, "Logical record length", 2, 0, 0, 0, &result)) {
        check_number_between_two_values(&blockette_pointer[11], 4, "Logical record length", 2, (double)8, (double)15, &result);
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked");
	add_error_to_result(&blockette_pointer[11], 4, "Logical record length", 2, 0, error_string, &result);
    }

    byte_number = 13;
    
    bytes_function_read = check_time_mask(&blockette_pointer[13], 5, "Beginning time", 22, &result);
    byte_number += bytes_function_read;

    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_8(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;

    int time1_length = 0;
    int time2_length = 0;

    char temp_version[5];
    char error_string[256];

    struct result_struct result;

    result.blockette_number = 8;
    strncpy(result.blockette_name, "Telemetry Volume Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 008", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;
    
    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 31, 115, &result);
    if (blockette_length < 31) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[7], 3, "Version of format", 4, 3, 0, 0, &result)) {
      if(check_number_between_two_values(&blockette_pointer[7], 3, "Version of format", 4, (double)0,(double)VERSION, &result) == 0) {
	strncpy(temp_version, &blockette_pointer[7], 4);
	temp_version[4] = '\0';
        version = atof(temp_version);
      }
      else {
        sprintf(error_string, "Because the value in the field was out of bounds,\n\t\tthe seed file is assumed to be version %f", (double) VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
      }
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked.\n\t\tThe version of the seed file is also assumed to be %f", (double)VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
    }

    if (check_number_mask(&blockette_pointer[11], 4, "Logical record length", 2, 0, 0, 0, &result)) {
        check_number_between_two_values(&blockette_pointer[11], 4, "Logical record length", 2, (double)8, (double)15, &result);
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked");
	add_error_to_result(&blockette_pointer[11], 4, "Logical record length", 2, 0, error_string, &result);
    }

    byte_number = 13;

    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 5, "Station identifier", 5, 0, "UN", &result);
    byte_number += bytes_function_read;
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 6, "Location identifier", 2, 0, "UN", &result);
    byte_number += bytes_function_read;
    bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 7, "Channel identifier", 3, 0, "UN", &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 8, "Beginning of volume", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 9, "End of volume", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[23], 8, "Beginning of vlume", time1_length, &blockette_pointer[23 + time1_length], 9, "End of volume", time2_length, &result);

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 10, "Station information effective date", 22, &result);
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 11, "Channel information effective date", 22, &result);
    byte_number += bytes_function_read;


    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
        print_errors(&result, verbose_option);
    }
    
    return(result.error);
}


int check_blockette_10(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;

    int time1_length = 0;
    int time2_length = 0;
    int time3_length = 0;

    char temp_version[5];
    char error_string[256];

    struct result_struct result;

    /* if the user entered a verbose_option > 0, this routine will
     * print out ERROR when it should be warning. Specifically for
     * volume time as compared to timespan time.
     * set to zero always.
     */

    result.blockette_number = 10;
    strncpy(result.blockette_name, "Volume Identifier Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;

    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 010", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;
    
    /*    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 15, 244, &result);*/
    if (blockette_length < 11) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    /*printf("(double)0: %f\t(double)VERSION: %f\n", (double)0, (double)VERSION);*/
    if (check_number_mask(&blockette_pointer[7], 3, "Version of format", 4, 3, 0, 0, &result)) {
      if(check_number_between_two_values(&blockette_pointer[7], 3, "Version of format", 4, (double)0,(double)VERSION, &result) == 0) {
	strncpy(temp_version, &blockette_pointer[7], 4);
	temp_version[4] = '\0';
        version = atof(temp_version);
      }
      else {
        sprintf(error_string, "Because the value in the field was out of bounds,\n\t\tthe seed file is assumed to be version %f", (double) VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
      }
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked.\n\t\tThe version of the seed file is also assumed to be %f", (double)VERSION);
	add_error_to_result(&blockette_pointer[byte_number], 3, "Version of format", 4, 0, error_string, &result);
	version = (double)VERSION;
    }

    if (version < 2.3) {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 17, 59, &result);
      if (blockette_length < 17) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }
    else {
      check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 22, 244, &result);
      if (blockette_length < 22) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
      }
    }


    if (check_number_mask(&blockette_pointer[11], 4, "Logical record length", 2, 0, 0, 0, &result)) {
        check_number_between_two_values(&blockette_pointer[11], 4, "Logical record length", 2, (double)8, (double)15, &result);
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field was not checked");
	add_error_to_result(&blockette_pointer[11], 4, "Logical record length", 2, 0, error_string, &result);
    }

    byte_number = 13;
    
    bytes_function_read = check_time_mask(&blockette_pointer[13], 5, "Beginning time", 22, &result);
    time1_length = bytes_function_read;
    byte_number += bytes_function_read;

    bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 6, "End time", 22, &result);
    time2_length = bytes_function_read;
    byte_number += bytes_function_read;

    check_two_times(&blockette_pointer[13], 5, "Beginning time", time1_length, &blockette_pointer[13 + time1_length], 6, "End time", time2_length, &result);

    /*if (byte_number == blockette_length) {
        if (result.error) {
	    print_errors(&result, verbose_option);
	}

	return(result.error);
    }*/


    if (version >= 2.3) {
      bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 7, "Volume time", 22, &result);
      time3_length = bytes_function_read;
      byte_number += bytes_function_read;

      check_two_times(&blockette_pointer[13], 5, "Beginning time", time1_length, &blockette_pointer[13 + time1_length + time2_length], 7, "Volume Time", time3_length, &result);

      check_two_times(&blockette_pointer[13 + time1_length], 
			6, "End time", 
			time2_length, 
			&blockette_pointer[13 + time1_length + time2_length], 
			7, 
			"Volume Time", 
			time3_length, &result);

      bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 8, "Originating Organization", 80, 1, "", &result);
      byte_number += bytes_function_read;

      if (byte_number < blockette_length) {
        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 9, "Label", 80, 1, "", &result);
	byte_number += bytes_function_read;
      }
    }


    if (byte_number != blockette_length) {
        print_blockette_length_problem(byte_number, blockette_length, &result); 
    }

    if (result.error) {
	/* we are always setting the verbose_option to zero so this routine
	 * will only print WARNINGS not ERRORS 
	 */
        print_errors(&result, 0);
    }

    return(result.error);
}


int check_blockette_11(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;

    int i;
    int number_of_stations;
    int error = 0;

    char error_string[256];

    struct result_struct result;

    result.blockette_number = 11;
    strncpy(result.blockette_name, "Volume Station Header Index Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;

    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 011", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);
   
    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 10, -1, &result);
    if (blockette_length < 10) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[7], 3, "Number of stations", 3, 0, 0, 0, &result)) {
        number_of_stations = return_number(&blockette_pointer[7], 3);

	if ((blockette_length - 10) % 11 != 0) {
	    sprintf(error_string, "The number of stations given in the field (%i) is wrong.\n\t\tFields 4 and 5 repeat %i times, while they should repeat %i times", number_of_stations, (blockette_length - 10) / 11, number_of_stations);
	    add_error_to_result(&blockette_pointer[7], 3, "Number of stations", 3, 1, error_string, &result);

	    sprintf(error_string, "Because of the above error, fields 4 and 5 were not checked in any way");
	    add_error_to_result(&blockette_pointer[7], 3, "Number of stations", 3, 1, error_string, &result);
	    error = 1;
	}

	if (!error) {
	    byte_number = 10;
	    for (i=0; i < number_of_stations; ++i) {
	        bytes_function_read = check_string_mask(&blockette_pointer[byte_number], 4, "Station identifier code", 5, 0, "UN", &result);
		byte_number += bytes_function_read;
		check_number_mask(&blockette_pointer[byte_number], 6, "Sequence number of station header", 6, 0, 0, 0, &result);
		byte_number += 6;
	    }
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field and the repeating fields 4 and 5 were not checked");
	add_error_to_result(&blockette_pointer[7], 3, "Number of stations", 3, 0, error_string, &result);
    }


    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}


int check_blockette_12(blockette_pointer, start_record, end_record, verbose_option)
char *blockette_pointer;
int start_record;
int end_record;
int verbose_option;
{
    int blockette_number;
    int blockette_length;
    int byte_number = 0;
    int bytes_function_read = 0;

    int i, j;
    int number_of_spans;

    int time1_length = 0;
    int time2_length = 0;

    char error_string[256];

    struct result_struct result;

    result.blockette_number = 12;
    strncpy(result.blockette_name, "Volume Time Span Index Blockette", 128);
    result.start_record_number = start_record;
    result.end_record_number = end_record;
    result.error = 0;
    result.mem_alloc_error = 0;
    result.next = NULL;


    /* get the blockette number and its length. Also check to make
       sure that the fields are valid. If they are not valid, take
       the apropriate action */

    blockette_number = return_number(blockette_pointer, 3);
    check_number_mask(blockette_pointer, 1, "Blockette type 012", 3, 0, 0, 0, &result);

    if (!check_number_mask(&blockette_pointer[3], 2, "Length of blockette", 4, 0, 0, 0, &result)) {
        sprintf(error_string, "Because there is a problem with the length filed, the rest\n\t\tthe rest of the blockette was not checked");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    blockette_length = return_number(&blockette_pointer[3], 4);

    byte_number = 7;

    check_length(&blockette_pointer[3], 2, "Lengh of blockette", 4, blockette_length, 11, -1, &result);
    if (blockette_length < 11) {
        sprintf(error_string, "Because the blockette is too short, it was not checked for errors");
	add_error_to_result(&blockette_pointer[3], 2, "Length of blockette", 4, 0, error_string, &result);
	print_errors(&result, verbose_option);
	return(result.error);
    }

    if (check_number_mask(&blockette_pointer[7], 3, "Number of spans in table", 4, 0, 0, 0, &result)) {
        number_of_spans = return_number(&blockette_pointer[7], 4);

	if (blockette_length > (number_of_spans * 52) + 11) {
	    sprintf(error_string, "The number of spans given in the field (%i) is less than the number indicated by the blockette size", number_of_spans);
	    add_error_to_result(&blockette_pointer[7], 3, "Number of spans", 3, 1, error_string, &result);
	}

	byte_number = 11;
	for (i=0; i < number_of_spans; ++i) {
	    if (byte_number + 23 < blockette_length) {
	        bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "Beginning of span", 22, &result);
		time1_length = bytes_function_read;
		byte_number += bytes_function_read;
	    }
	    else {
	        for (j=0; j < blockette_length - byte_number && blockette_pointer[j] != '~'; ++j) {
		    if (blockette_pointer[j] == '~') {
		        --j;
			break;
		    }
		}
		   
	        bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "Beginning of span", j, &result);
		time1_length = bytes_function_read;
		byte_number += bytes_function_read;
	    }

	    if (byte_number + 23 < blockette_length) {
	        bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 5, "End of span", 22, &result);
		time2_length = bytes_function_read;
		byte_number += bytes_function_read;
	    }
	    else {
	        for (j=0; j < blockette_length - byte_number && blockette_pointer[j] != '~'; ++j) {
		    if (blockette_pointer[j] == '~') {
		        --j;
			break;
		    }
		}
		   
	        bytes_function_read = check_time_mask(&blockette_pointer[byte_number], 4, "End of span", j, &result);
		time2_length = bytes_function_read;
		byte_number += bytes_function_read;
	    }

	    check_two_times(&blockette_pointer[byte_number - time1_length - time2_length], 4, "Beginning of span", time1_length, &blockette_pointer[byte_number - time2_length], 5, "End of span", time2_length, &result);

	    if (byte_number + 6 <= blockette_length) {
	        check_number_mask(&blockette_pointer[byte_number], 6, "Sequence number of time span header", 6, 0, 0, 0, &result);
		byte_number += 6;
	    }
	    else {
	        sprintf(error_string, "The blockette ended before span number %i had a chance to complete", i);
		add_error_to_result(&blockette_pointer[7], 3, "Number of spans", 4, 1, error_string, &result);
	    }
	}

	if (byte_number != blockette_length) {
	    print_blockette_length_problem(byte_number, blockette_length, &result); 
	}
    }
    else {
        sprintf(error_string, "Because the value in the field did not conform to the mask,\n\t\tthe value of the field and repeating fields 4, 5, and 6 were not checked");
	add_error_to_result(&blockette_pointer[7], 3, "Number of spans in table", 4, 0, error_string, &result);
    }


    if (result.error) {
        print_errors(&result, verbose_option);
    }

    return(result.error);
}
