/*===========================================================================*/
/* SEED reader     |              parse_double             |         utility */
/*===========================================================================*/
/*
	Name:		parse_double
	Purpose:	parse a double (float) field from a blockette
	Usage:		double parse_double ();
				char *blockette_ptr;
				int number_of_bytes;
				double result;
				result = parse_double (&blockette_ptr, number_of_bytes);
	Input:		blockette_ptr = ptr to the blockette_ptr
				number_of_bytes = number of bytes from blockette to process
	Output:		blockette_ptr = points to beginning of next string
				result = the value of the double field
	Externals:	temp_char - allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	any parsing procedure
	Calls to:	memncpy - does what strncpy is alleged to do
	Algorithm:	get number_of_bytes bytes from the blockette; increment the
				blockette pointer; return the double value of the string 
				to the caller
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/21/88  Dennis O'Neill  Production release 1.0
*/

#include <math.h>
#include <stdlib.h>
#include "rdseed.h"

double parse_double (blockette_ptr, number_of_bytes)
char **blockette_ptr;
int number_of_bytes;
{
	double mantissa, exp;

	memcpy (temp_char, *blockette_ptr, number_of_bytes); 
	temp_char[number_of_bytes] = 0;

	/* scan for spaces at end */
	trim(temp_char);

    *blockette_ptr += number_of_bytes;

	if (strstr(temp_char, "E") != NULL)
	{
		/* make sure it is in the right position - 4 from the end */

		if (temp_char[strlen(temp_char) - 4] != 'E')
		{
			fprintf(stderr, "Warning, bad floating point number! Float=%s\n", temp_char);
		}

	}

    if (strstr(temp_char, "E ") != NULL)    /* catch 'funny' conditions     JAL001 */
    {
        sscanf(temp_char, "%lfE %lf", &mantissa, &exp);
        return( mantissa * pow(10.0, exp));
    }
    else if (strstr(temp_char, "e ") != NULL)
    {
        sscanf(temp_char, "%lfe %lf", &mantissa, &exp);
        return( mantissa * pow(10.0, exp));
    }
    else
    {   
        return (atof (temp_char));
    }

}
